#include <zephyr/kernel.h>
#include <zephyr/device.h>
#include <zephyr/drivers/uart.h>
#include <zephyr/sys/printk.h>
#include <string.h>
#include <stdio.h>

//#define UART_DEVICE_NODE DT_NODELABEL(uart1) // Update based on your hardware configuration

//static const struct device *uart_dev;
const struct device *uart_dev = DEVICE_DT_GET(DT_NODELABEL(uart1));


#define TX_BUFFER_SIZE 64
#define RX_BUFFER_SIZE 64

static uint8_t tx_buffer[TX_BUFFER_SIZE];
static uint8_t rx_buffer[RX_BUFFER_SIZE]={0};

static volatile bool tx_done = false;
static volatile bool rx_ready = false;

struct uart_config uart_cfg = {
    .baudrate = 9600,
    .data_bits = UART_CFG_DATA_BITS_8,
    .parity = UART_CFG_PARITY_NONE,
    .stop_bits = UART_CFG_STOP_BITS_1,
    .flow_ctrl = UART_CFG_FLOW_CTRL_NONE,

};


// UART callback function
void uart_cb(const struct device *dev, struct uart_event *evt, void *user_data) {

    int err;
    
    switch (evt->type) {
    case UART_TX_DONE:
        printk("Transmission complete\n");
        tx_done = true;
            err = uart_rx_enable(uart_dev, rx_buffer, sizeof(rx_buffer), 50); // Timeout in ms
    if (err) {
        printk("Failed to enable UART RX: %d\n", err);
        return;
    }
        break;

    case UART_RX_RDY:
        printk("Received data: ");
        for (int i = evt->data.rx.offset; i < evt->data.rx.offset + evt->data.rx.len; i++) {
            printk("%c", rx_buffer[i]);
        }
        printk("\n");
        rx_ready = true;
        break;

    case UART_RX_DISABLED:
        printk("UART RX disabled\n");
        break;

    case UART_RX_BUF_REQUEST:
        // Provide a new buffer for UART reception
        uart_rx_buf_rsp(dev, rx_buffer, sizeof(rx_buffer));
        break;

    default:
        printk("Unhandled UART event: %d\n", evt->type);
        break;
    }
}

// Initialize UART
void uart_init(void) {

int rc;

    if (!uart_dev) {
        printk("Failed to get UART device\n");
        return;
    }


rc = uart_configure(uart_dev, &uart_cfg);
if (rc != 0) {
        printf("Could not configure device uart1\n");
} else {
        printf("Configure device uart1\n");
}


if (!device_is_ready(uart_dev)) 
{
    printf("UART1 device not ready\n");
} else {
    printf("UART1 device ready\n");

}
    // Set the UART callback
    int err = uart_callback_set(uart_dev, uart_cb, NULL);
    if (err) {
        printk("Failed to set UART callback: %d\n", err);
        return;
    }

    // Enable RX
    err = uart_rx_enable(uart_dev, rx_buffer, sizeof(rx_buffer), 50); // Timeout in ms
    if (err) {
        printk("Failed to enable UART RX: %d\n", err);
        return;
    }

    printk("UART initialized successfully\n");
}

// Transmit string over UART
void uart_transmit(const char *data) {
    size_t len = strlen(data);

    if (len > TX_BUFFER_SIZE) {
        printk("Data too large to send\n");
        return;
    }

    memcpy(tx_buffer, data, len);

    int err = uart_tx(uart_dev, tx_buffer, len, SYS_FOREVER_MS);
    if (err) {
        printk("UART TX failed: %d\n", err);
        return;
    }

    // Wait until transmission is complete
    while (!tx_done) {
        k_sleep(K_MSEC(10));
    }
    tx_done = false;
}

int main(void) {
    uart_init();

    for (int i = 0; i < 15; i++) {
        // Get current time (simulated with seconds since boot)
        uint32_t time_ms = k_uptime_get_32() / 1000; // Convert ms to seconds
        char time_str[64];
        snprintf(time_str, sizeof(time_str), "Time: %lu seconds\n", (unsigned long)time_ms);

        // Transmit time string over UART
        uart_transmit(time_str);

        // Print status to console
        printk("Sent: %s", time_str);

        // Wait for 1 second
        k_sleep(K_SECONDS(1));
    }

    printk("Finished transmitting time strings\n");

    // Keep application alive to allow RX events to be handled
    while (1) {
        if (rx_ready) {
            printk("Processing received data...\n");
            rx_ready = false;
        }
        k_sleep(K_MSEC(100));
    }
}
